/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.platform;

import java.util.Collection;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.create.Creates;
import org.fortiss.af3.exploration.alg.dse.module.CreatorModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IGatewayUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IMemoryUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceConnectionAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceConnectionAdapter.ConnectionType;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITransmissionUnitAdapter;
import org.jgrapht.Graphs;
import org.jgrapht.graph.DefaultDirectedGraph;
import org.jgrapht.graph.DefaultEdge;

import com.google.inject.Provides;

/**
 * Creator of {@link PlatformCommunicationGraphEncoding}s.
 * 
 * @author diewald
 */
public class PlatformCommunicationGraphCreator extends
		CreatorModule<PlatformCommunicationGraphEncoding> {

	/** Creates a {@link PlatformCommunicationGraphEncoding} from the input model. */
	@Provides
	@Creates
	public PlatformCommunicationGraphEncoding create(
			Collection<IExecutionUnitAdapter<?>> execUnits,
			Collection<IMemoryUnitAdapter<?>> memoryUnits,
			Collection<IGatewayUnitAdapter<?>> gatewayUnits,
			Collection<ITransmissionUnitAdapter<?>> transmissionUnits,
			Collection<IResourceConnectionAdapter> resourceConnections) {
		DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge> tempPlatformCommGraph =
				new DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge>(DefaultEdge.class);
		DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge> platformGraph =
				new DefaultDirectedGraph<IResourceAdapter<?>, DefaultEdge>(DefaultEdge.class);

		// Add all resources present in the platform as vertices to the "connection graph"
		Graphs.addAllVertices(tempPlatformCommGraph, execUnits);
		Graphs.addAllVertices(tempPlatformCommGraph, memoryUnits);
		Graphs.addAllVertices(tempPlatformCommGraph, gatewayUnits);
		Graphs.addAllVertices(tempPlatformCommGraph, transmissionUnits);

		// Successively connect the resources in the graph. The edges are given within the
		// collection resource connectors are assumed to be fully connected.
		for(IResourceConnectionAdapter connection : resourceConnections) {
			// We add edges for each receiver in order to support broadcast connections
			for(IResourceAdapter<?> currentReceiverResource : connection.getTargetResources()) {
				// If the current resource is a child of a {@link TransmissionUnit} (like a NoC
				// router), respect the direction of connections such that complex networks are
				// correctly reflected.
				if(connection.getConnectionType(currentReceiverResource) == ConnectionType.OUTGOING) {
					Graphs.addEdgeWithVertices(platformGraph, connection.getSourceResource(),
							currentReceiverResource);
				} else if(connection.getConnectionType(currentReceiverResource) == ConnectionType.INCOMING) {
					Graphs.addEdgeWithVertices(platformGraph, currentReceiverResource,
							connection.getSourceResource());
				} else {
					// Add the edges of a single connection. We add to edges for each connection
					// such that an undirected graph is generated effectively. It is required to use
					// this "trick" in order to be able to use the standard shortest path
					// algorithms.
					Graphs.addEdgeWithVertices(platformGraph, connection.getSourceResource(),
							currentReceiverResource);
					Graphs.addEdgeWithVertices(platformGraph, currentReceiverResource,
							connection.getSourceResource());
				}
			}
		}
		return new PlatformCommunicationGraphEncoding(platformGraph);
	}

	/** {@inheritDoc} */
	@Override
	public PlatformCommunicationGraphEncoding create() {
		// TODO Auto-generated method stub
		return null;
	}
}
