/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskgraph.AbstractTaskGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.abstractmapping.AbstractTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.exception.ExplorationException;

/**
 * Problem encoding of redundant {@link TaskMappingEncoding}s with fault detection and voting.
 * 
 * @author huang
 */
public class FaultDetectionVotingTaskMappingEncoding extends
		InstantiatedTaskMappingEncoding<FaultDetectionVotingTaskMappingEntry> implements Cloneable {

	/**
	 * Constructor that is initialized with a reference to the DSE specification and the input
	 * system model.
	 */
	FaultDetectionVotingTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding tgEncoding) {
		super(partEnc, tgEncoding);
	}

	// TODO: Check whether the two constructors below are actually needed.
	/**
	 * Constructor that optionally initializes the encoding to a random encoding
	 * 
	 * @throws ExplorationException
	 *             if no valid allocation targets can be identified for one {@link ITaskAdapter}.
	 */
	public FaultDetectionVotingTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding tgEncoding, boolean initialize) throws ExplorationException {
		super(partEnc, tgEncoding, initialize);
		assert (false) : "Never here any more.";
	}

	/**
	 * Constructor which derives the resulting {@link FaultDetectionVotingTaskMappingEncoding} from
	 * the given {@link AbstractTaskMappingEncoding}.
	 */
	public FaultDetectionVotingTaskMappingEncoding(PartitionMappingEncoding partEnc,
			AbstractTaskGraphEncoding tgEncoding, AbstractTaskMappingEncoding abstractEncoding) {
		super(partEnc, tgEncoding, abstractEncoding);
	}

	/** {@inheritDoc} */
	@Override
	public FaultDetectionVotingTaskMappingEntry createEntry(ITaskAdapter<?> deployableComponent,
			Partition deploymentTarget) {
		if(deployableComponent == null || deploymentTarget == null) {
			return null;
		}

		return new FaultDetectionVotingTaskMappingEntry(deployableComponent.clone(),
				deploymentTarget, tgEncoding.getTaskGraphOf(deployableComponent)
						.getRandomFaultDetectorId(deployableComponent));
	}

	/** {@inheritDoc} */
	@Override
	public <S extends TaskMappingEntry<ITaskAdapter<?>, Partition>>
			FaultDetectionVotingTaskMappingEntry createEntry(S mapping) {
		// TODO: Do not use 0, magic constant.
		return new FaultDetectionVotingTaskMappingEntry((AbstractTaskMappingEntry)mapping, 0);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public FaultDetectionVotingTaskMappingEncoding newInstance() {
		try {
			return new FaultDetectionVotingTaskMappingEncoding(null, null, true);
		} catch(ExplorationException e) {
			// Opt4J does not allow to throw Exceptions here.
			throw new RuntimeException(e);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public FaultDetectionVotingTaskMappingEncoding copy() {
		// TODO: Here, we should maintain the mapping.
		FaultDetectionVotingTaskMappingEncoding clonedEncoding = null;
		try {
			clonedEncoding = new FaultDetectionVotingTaskMappingEncoding(null, null, false);
		} catch(ExplorationException e) {
			// Opt4J does not allow to throw Exceptions here.
			throw new RuntimeException(e);
		}
		return clonedEncoding;
	}
}
