/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ThreadScope;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.ThreadScoped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.Genotype;
import org.opt4j.core.Individual;
import org.opt4j.core.problem.Evaluator;

import com.google.inject.AbstractModule;
import com.google.inject.Injector;
import com.google.inject.Provides;

/**
 * This module enables an {@link Evaluator} instance to obtain an evaluator and thread-specific
 * {@link Injector}. This {@link Injector} carries the data containers of {@link Genotype}s and
 * {@link Phenotype}s and provides them to the corresponding processing thread of the
 * {@link Individual}.
 * 
 * @author diewald
 */
public class EvaluatorInjectorProvider extends AbstractModule {

	/** Thread/{@link Individual}-specific {@link Injector}. */
	private Injector evalInjector;

	/** Provides a Thread/{@link Individual}-specific {@link EvaluatorInjectorProvider} instance. */
	@Provides
	@ThreadScoped
	public EvaluatorInjectorProvider getEvalutatorInjectorProvider() {
		return new EvaluatorInjectorProvider();
	}

	/** Sets the Thread/{@link Individual}-specific {@link Injector}. */
	public void setEvalInjector(Injector evalInjector) {
		this.evalInjector = evalInjector;
	}

	/** Returns Thread/{@link Individual}-specific {@link Injector}. */
	public Injector getEvalInjector() {
		return evalInjector;
	}

	/** {@inheritDoc} */
	@Override
	protected void configure() {
		bindScope(ThreadScoped.class, new ThreadScope());
	}
}
