/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype;

import java.util.Map.Entry;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.EncodingProviderModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;

import com.google.common.collect.ClassToInstanceMap;
import com.google.common.collect.MutableClassToInstanceMap;

/**
 * {@link CompositeGenotype} that additionally holds an {@link EncodingProviderModule} which
 * provides access to the {@link Genotype}s by Guice @Provides methods.
 * 
 * @author diewald
 */
public class ProvidingGenotype extends CompositeGenotype<Class<? extends Genotype>, Genotype> {

	/**
	 * Holds the reference to the {@link EncodingProviderModule} provided along this
	 * {@link CompositeGenotype}.
	 */
	private EncodingProviderModule<Genotype> providerModule;

	/** References the map of solution encodings. */
	// TODO: This field is for legacy support (transformation). We can switch the transformation
	// mechanism to use DI as well.
	private ClassToInstanceMap<Genotype> genotypeMap = MutableClassToInstanceMap.create();

	/** Constructor. */
	public ProvidingGenotype(EncodingProviderModule<Genotype> providerModule,
			CompositeGenotype<Class<? extends Genotype>, Genotype> compositeGenotype) {
		this.providerModule = providerModule;
		for(Entry<Class<? extends Genotype>, Genotype> entry : compositeGenotype) {
			map.put(entry.getKey(), entry.getValue());
			genotypeMap.put(entry.getKey(), entry.getValue());
		}
	}

	/**
	 * Returns the {@link EncodingProviderModule} that is passed along this
	 * {@link CompositeGenotype}.
	 */
	public EncodingProviderModule<Genotype> getProviderModule() {
		return providerModule;
	}

	/**
	 * Returns the map of {@link Genotype}s used to construct the {@link Phenotype}s of this
	 * solution candidate.
	 */
	public ClassToInstanceMap<Genotype> getGenotypeMap() {
		return this.genotypeMap;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public ProvidingGenotype newInstance() {
		return new ProvidingGenotype(this.providerModule, this);
	}
}
