/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.InstantiatedTaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.evaluator.GenericValue;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.ComponentMultiDislocationConstraint;
import org.opt4j.core.Value;

import com.google.inject.Provides;
import com.google.inject.name.Named;

/**
 * Constraint which defines the disallowed allocations of {@link ITaskAdapter}s to
 * {@link IExecutionUnitAdapter}s.
 * 
 * @author diewald
 */
// TODO: Register this evaluator for AstractTaskMappings.
public class ComponentMultiDislocationConstraintEvaluator<S extends InstantiatedTaskMappingEntry, T extends InstantiatedTaskMappingEncoding<S>>
		extends ConstraintEvaluatorBase<Boolean, ComponentMultiDislocationConstraint, T> {

	/** Constructor. */
	public ComponentMultiDislocationConstraintEvaluator(
			Class<ComponentMultiDislocationConstraint> explorationTargetType) throws Exception {
		super(explorationTargetType);
	}

	/** Constructor. */
	public ComponentMultiDislocationConstraintEvaluator(
			ComponentMultiDislocationConstraint explorationTarget) throws Exception {
		super(explorationTarget);
	}

	/** {@inheritDoc} */
	@Provides
	@Named("ComponentMultiDislocationConstraintEvaluator")
	public GenericValue<Boolean> evaluate() {
		// Already considered in the operators.
		return new GenericValue<Boolean>(true) {
			@Override
			public Double getDouble() {
				return 0D;
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	public boolean validate(ExplorationConstraint<?> expTarget, T solution, Value<?> value) {
		// Already considered in the operators.
		// TODO: Just to be safe: add a real check.
		return true;
	}
}
