/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.opt4j.core.Value;

/**
 * Specifies methods relevant for the handling of {@link ExplorationConstraint}s in
 * {@link EvaluatorModule}s.
 * 
 * @author diewald
 */
public interface IConstraintEvaluator<R, T extends ExplorationConstraint<R>, P extends Phenotype> {

	/**
	 * Validates whether the given constraint is satisfied. This method is called for each produced
	 * {@code solution} to generate only feasible solutions once the stopping criterion is reached.
	 */
	boolean validate(ExplorationConstraint<?> expTarget, P solution, Value<?> value);
}
