/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;
import static org.fortiss.tooling.common.util.LambdaUtils.getFirst;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.instantiatedmapping.FailSilentTaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;
import org.fortiss.af3.platform.model.allocation.ComponentToExecutionUnitAllocationEntry;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Transforms a {@link TaskMappingEncoding} and a {@link PartitionMappingEncoding} to a
 * {@link SuperSet} of AF3 {@link ComponentToExecutionUnitAllocationEntry}s.
 * 
 * @author diewald
 */
public class ComponentExecutionUnitTransformation extends
		SuperSetTransformationBase<ComponentToExecutionUnitAllocationEntry> {
	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> SuperSet<ComponentToExecutionUnitAllocationEntry>
			transform(TransformationState state, TransformationContext context,
					ExplorationTransformationInputs inputs,
					SubClassToInstanceMap<N> transformedModels, EObject outputModel) {
		FailSilentTaskMappingEncoding tm = inputs.getEncoding(FailSilentTaskMappingEncoding.class);
		PartitionMappingEncoding pmEncoding = inputs.getEncoding(PartitionMappingEncoding.class);

		SuperSet<ComponentToExecutionUnitAllocationEntry> compExecUnitAllocs =
				createSuperSet("Optimized_Allocations",
						ComponentToExecutionUnitAllocationEntry.class);
		for(Partition resource : tm.getResources()) {
			for(ITaskAdapter<?> task : tm.getRequesters(resource)) {
				Component af3Component = task.getAssignedComponent();
				ComponentToExecutionUnitAllocationEntry allocEntry =
						createAllocationEntry(ComponentToExecutionUnitAllocationEntry.class);
				allocEntry.getSourceElements().add(af3Component);
				allocEntry.setTargetElement((IModelElement)getFirst(
						pmEncoding.getAllocatedResourcesFor(resource)).get().getObject());
				compExecUnitAllocs.getEntries().add(allocEntry);
			}
		}
		return compExecUnitAllocs;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(FailSilentTaskMappingEncoding.class);
		inputTypes.add(PartitionMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<ComponentToExecutionUnitAllocationEntry> getOutputType() {
		return ComponentToExecutionUnitAllocationEntry.class;
	}
}
