/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import org.fortiss.af3.exploration.alg.dse.modeltransformation.ITransformationModule;
import org.fortiss.af3.exploration.alg.dse.modeltransformation.base.TransformationModuleBase;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;
import org.fortiss.tooling.base.model.element.IModelElement;

/**
 * Base class for transforming inner solution representations of the DSE to {@link SuperSet}s.
 * 
 * @author diewald
 */
public abstract class SuperSetTransformationBase<T extends IModelElement>
		extends TransformationModuleBase<SuperSet<T>> {
	/** {@inheritDoc} */
	@Override
	public boolean isOptional() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * NOTE: Currently, we neglect states in exporting DSE results: Transformations to SuperSets are
	 * typically simple, so stages are typically adding unneeded complexity.
	 */
	@Override
	public TransformationState getActivationStage() {
		return TransformationState.INITIALIZING;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * NOTE: Currently, we neglect hierarchies in exporting DSE results.
	 */
	@Override
	public Collection<Class<? extends ITransformationModule<?>>> getParentModules() {
		Collection<Class<? extends ITransformationModule<?>>> parentModules = new ArrayList<>();
		parentModules.add(SuperSetTransformationStrategy.class);
		return parentModules;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getOptionalTypes() {
		return Collections.emptySet();
	}
}
