/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.feature;

import static java.util.Arrays.asList;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.partitionmapping.PartitionMappingCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.create.platform.PlatformCommunicationGraphCreator;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.platform.PlatformCommunicationGraphEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.partitionmapping.PartitionMappingModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.mutate.platform.NopPlatformCommGraphMutateModule;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.problem.partitionmapping.PartitionMappingDecoderGraph;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.NoEmptyPartitionConstraintEvaluator;
import org.fortiss.af3.exploration.alg.dse.evaluator.constraint.SameSilInPartitionConstraintEvaluator;
import org.fortiss.af3.exploration.moea.model.predefined.NoEmptyPartitionConstraint;
import org.fortiss.af3.exploration.moea.model.predefined.SameSilInPartitionConstraint;

import com.google.inject.Injector;

/**
 * Adds a virtualization layer to the mapping problem such that an optimal number of
 * {@link Partition}s is found that provide time/space separation.
 * 
 * @author diewald
 */
public class PartitionArchitectureExploration extends ExplorationFeature {

	/** Constructor. */
	public PartitionArchitectureExploration() {
		super("Partition Architecture Exploration");

		addVariable(PartitionMappingEncoding.class, new PartitionMappingCreator(),
				asList(new PartitionMappingModule()));
		addVariable(PlatformCommunicationGraphEncoding.class,
				new PlatformCommunicationGraphCreator(),
				asList(new NopPlatformCommGraphMutateModule()));
		addProblem(PlatformCommunicationGraphEncoding.class, new PartitionMappingDecoderGraph());
		addEvaluator(new NoEmptyPartitionConstraintEvaluator(NoEmptyPartitionConstraint.class));
		addEvaluator(new SameSilInPartitionConstraintEvaluator(SameSilInPartitionConstraint.class));
	}

	/** {@inheritDoc} */
	@Override
	public boolean isApplicableToInputs(Injector dseInjector) {
		return true;
	}
}
